import java.util.*;

public class Payroll
{
	private Map hourlyEmp; //TreeMap for Hourly Employees - key is name
	private Map salariedEmp; //TreeMap for Salaried Employees - key is name
	private Map allEmp;  //HashMap for all employees - key is id
	
	public Payroll()
	{
		hourlyEmp = new TreeMap();
		salariedEmp = new TreeMap();
		allEmp = new HashMap();
	}
	
	//emp has been added to hourlyEmp and allEmp
	public void addEmp(HourlyEmployee emp)
	{
		hourlyEmp.put(emp.getName(), emp);
		allEmp.put(emp.getID(), emp);	
	}	

	//emp has been added to salariedEmp and allEmp
	public void addEmp(SalariedEmployee emp)
	{
		salariedEmp.put(emp.getName(), emp);
		allEmp.put(emp.getID(), emp);	
	}
	
	//precondition: if an employee with id employee id
	//exists, that employee is either in hourlyEmp or
	//salariedEmp, not both.
	public void removeEmp(String id)
	{
		Employee emp = (Employee) allEmp.remove(id);
		if(emp != null)
		{
			if(emp.equals(hourlyEmp.get(emp.getName())))
			    hourlyEmp.remove(emp.getName());
			else
			    salariedEmp.remove(emp.getName());
		}
	}
	
	//precondition:  t holds all the timecards for hourly
	//employees.
	//postcondition:  all hourly employees' weekly hours
	//worked has been updated
	public void updateWeeklyHours(TimeCards t)
	{
		Set keys = hourlyEmp.keySet();
		Iterator itr = keys.iterator();
		while(itr.hasNext())
		{
			HourlyEmployee emp = (HourlyEmployee) hourlyEmp.get(itr.next());
			emp.setHoursWorked(t.getHoursWorked(emp.getID()));
		}
	}
	
	
 
	//returns the total amount of all employee's paychecks
	public double getTotalPayroll()
	{
		Set s = allEmp.keySet();
		Iterator itr = s.iterator();
		double total = 0;
		while(itr.hasNext())
		{
			Employee emp = (Employee)allEmp.get(itr.next());
			total += emp.getPayCheck();
		}
		return total;	
	}
	
	public Employee getEmployee(String id)
	{
		return (Employee) allEmp.get(id);
	}
	
	public void printHourlyEmployees()
	{
		print(hourlyEmp,"Hourly");
	}
	
	public void printSalariedEmployees()
	{
		print(salariedEmp, "Salaried");
	}
	
	private void print(Map m, String type)
	{
		Iterator itr = m.keySet().iterator();
		System.out.println("================================================");		
		System.out.println("Listing of all " + type + " Employees");
		while(itr.hasNext())
		{
			System.out.println(m.get(itr.next()) + "\n");
		}	
		System.out.println("================================================\n");
	}			
}
